"use client";
import { useLayoutEffect } from "react";
import { f as createTemplate, e as getLineBefore, h as getLineEnd, j as scrollToEl } from "../index-cQWT8Bdc.js";
import { a as addTextareaListener } from "../core-D93zuxUs.js";
const cursorTemplate = createTemplate(
  "<div style=position:absolute;top:0;opacity:0;padding:inherit> <span><span></span> "
);
const useCursorPosition = (editor) => {
  useLayoutEffect(() => {
    let prevBefore = " ";
    let prevAfter = " ";
    const cursorContainer = cursorTemplate();
    const [before, span] = cursorContainer.childNodes;
    const [cursor, after] = span.childNodes;
    const selectionChange = (selection) => {
      let value = editor.value;
      let activeLine = editor.lines[editor.activeLine];
      let position = selection[selection[2] < "f" ? 0 : 1];
      let newBefore = getLineBefore(value, position);
      let newAfter = value.slice(position, getLineEnd(value, position));
      if (!newBefore && !newAfter)
        newAfter = "\n";
      if (prevBefore != newBefore)
        before.data = prevBefore = newBefore;
      if (prevAfter != newAfter)
        after.data = prevAfter = newAfter;
      if (cursorContainer.parentNode != activeLine)
        activeLine.prepend(cursorContainer);
    };
    const scrollIntoView = () => scrollToEl(editor, cursor);
    const cleanup1 = editor.on("selectionChange", selectionChange);
    const cleanup2 = addTextareaListener(editor, "input", (e) => {
      if (/history/.test(e.inputType))
        scrollIntoView();
    });
    editor.extensions.cursor = {
      scrollIntoView,
      getPosition() {
        const rect1 = cursor.getBoundingClientRect();
        const rect2 = editor.lines[0].getBoundingClientRect();
        return {
          top: rect1.y - rect2.y,
          bottom: rect2.bottom - rect1.bottom,
          left: rect1.x - rect2.x,
          right: rect2.right - rect1.x,
          height: rect1.height
        };
      }
    };
    return () => {
      delete editor.extensions.cursor;
      cleanup1();
      cleanup2();
      cursor.remove();
    };
  }, []);
};
export {
  useCursorPosition
};
//# sourceMappingURL=cursor.js.map
