import { Token } from '../../prism';
import { PrismEditor } from '../../types';

export interface BracketMatcher {
    /**
     * Array of tuples containing in the following order:
     * - The bracket's `Token`
     * - Its starting position
     * - Its level of nesting
     * - Its text content
     * - Whether it's an opening bracket
     * - Its ending position
     *
     * The order will likely change in the next major release
     */
    readonly brackets: Bracket[];
    /** Array mapping the index of a bracket to the index of its matching bracket. */
    readonly pairs: (number | undefined)[];
}
/**
 * Tuple containing in the following order:
 * - The bracket's `Token`
 * - Its starting position
 * - Its level of nesting
 * - Its text content
 * - Whether it's an opening bracket
 * - Its ending position
 *
 * The order will likely change in the next major release
 */
export type Bracket = [Token, number, number, string, boolean, number];
/**
 * Hook that matches punctuation tokens together. Intended for matching brackets.
 *
 * The order inside `openingBrackets` and `closingBrackets` determines which characters
 * are matched together.
 * @param rainbowBrackets Whether to add extra classes to brackets for styling. Defaults to `true`.
 * @param openingBrackets Defaults to `"([{"`.
 * @param closingBrackets Defaults to `")]}"`.
 */
export declare const useBracketMatcher: (editor: PrismEditor, rainbowBrackets?: boolean, openingBrackets?: string, closingBrackets?: string) => void;
