import { n as numLines, a as addTextareaListener, s as selectionChange } from "./core-D93zuxUs.js";
const scrollToEl = (editor, el, paddingTop = 0) => {
  const style1 = editor.container.style;
  const style2 = document.documentElement.style;
  style1.scrollPaddingBlock = style2.scrollPaddingBlock = `${paddingTop}px ${isChrome && !el.textContent ? el.offsetHeight : 0}px`;
  el.scrollIntoView({ block: "nearest" });
  style1.scrollPaddingBlock = style2.scrollPaddingBlock = "";
};
const getLineStart = (text, position) => position ? text.lastIndexOf("\n", position - 1) + 1 : 0;
const getLineEnd = (text, position) => (position = text.indexOf("\n", position)) + 1 ? position : text.length;
const templateEl = /* @__PURE__ */ document.createElement("div");
const createTemplate = (html) => {
  templateEl.innerHTML = html;
  const node = templateEl.firstChild;
  return () => node.cloneNode(true);
};
let prevSelection;
const regexEscape = (str) => str.replace(/[$+?|.^*()[\]{}\\]/g, "\\$&");
const getLineBefore = (text, position) => text.slice(getLineStart(text, position), position);
const getLines = (text, start, end = start) => [
  text.slice(start = getLineStart(text, start), end = getLineEnd(text, end)).split("\n"),
  start,
  end
];
const getClosestToken = (editor, selector, marginLeft = 0, marginRight = marginLeft, position = editor.getSelection()[0]) => {
  const value = editor.value;
  const line = editor.lines[numLines(value, 0, position)];
  const walker = document.createTreeWalker(line, 5);
  let node = walker.lastChild();
  let offset = getLineEnd(value, position) + 1 - position - node.length;
  while (-offset <= marginRight && (node = walker.previousNode())) {
    if (node.lastChild)
      continue;
    offset -= node.length || 0;
    if (offset <= marginLeft) {
      for (; node != line; node = node.parentNode) {
        if (node.matches?.(selector))
          return node;
      }
    }
  }
};
const getLanguage = (editor, position) => getClosestToken(editor, '[class*="language-"]', 0, 0, position)?.className.match(
  /language-(\S*)/
)[1] || editor.props.language;
const insertText = (editor, text, start, end, newCursorStart, newCursorEnd) => {
  if (!editor.props.readOnly) {
    prevSelection = editor.getSelection();
    end ??= start;
    let textarea = editor.textarea;
    let value = editor.value;
    let avoidBug = isChrome && !value[end ?? prevSelection[1]] && /\n$/.test(text) && /^$|\n$/.test(value);
    editor.focused || textarea.focus();
    if (start != null)
      textarea.setSelectionRange(start, end);
    if (newCursorStart != null) {
      addTextareaListener(
        editor,
        "input",
        () => {
          textarea.setSelectionRange(
            newCursorStart,
            newCursorEnd ?? newCursorStart,
            prevSelection[2]
          );
        },
        { once: true, capture: true }
      );
    }
    isWebKit || textarea.dispatchEvent(new InputEvent("beforeinput", { data: text }));
    if (isChrome || isWebKit) {
      if (avoidBug) {
        textarea.selectionEnd--;
        text = text.slice(0, -1);
      }
      if (isWebKit)
        text += "\n";
      document.execCommand(
        text ? "insertHTML" : "delete",
        false,
        text.replace(/&/g, "&amp;").replace(/</g, "&lt;")
      );
      if (avoidBug)
        textarea.selectionStart++;
    } else
      document.execCommand(text ? "insertText" : "delete", false, text);
    prevSelection = 0;
  }
};
const getModifierCode = (e) => e.altKey + e.ctrlKey * 2 + e.metaKey * 4 + e.shiftKey * 8;
const userAgent = navigator.userAgent;
const isMac = /Mac|iPhone|iPod|iPad/i.test(navigator.platform);
const isChrome = /Chrome\//.test(userAgent);
const isWebKit = !isChrome && /AppleWebKit\//.test(userAgent);
const setSelection = (editor, start, end = start, direction) => {
  let focused = editor.focused;
  let textarea = editor.textarea;
  let relatedTarget;
  if (!focused) {
    addTextareaListener(
      editor,
      "focus",
      (e) => {
        relatedTarget = e.relatedTarget;
      },
      { once: true }
    );
    textarea.focus();
  }
  textarea.setSelectionRange(start, end, direction);
  selectionChange(!(!focused && (relatedTarget ? relatedTarget.focus() : textarea.blur())));
};
export {
  getLines as a,
  getLineStart as b,
  getLanguage as c,
  insertText as d,
  getLineBefore as e,
  createTemplate as f,
  getModifierCode as g,
  getLineEnd as h,
  isMac as i,
  scrollToEl as j,
  getClosestToken as k,
  isWebKit as l,
  isChrome as m,
  prevSelection as p,
  regexEscape as r,
  setSelection as s
};
//# sourceMappingURL=index-cQWT8Bdc.js.map
