import { useMemo, useLayoutEffect, useEffect } from "react";
import { a as addTextareaListener, p as preventDefault, n as numLines } from "./core-D93zuxUs.js";
import { f as createTemplate, r as regexEscape, s as setSelection, j as scrollToEl, d as insertText, i as isMac, g as getModifierCode, l as isWebKit, b as getLineStart, h as getLineEnd } from "./index-cQWT8Bdc.js";
import { g as getStyleValue } from "./other-EdiSn7BB.js";
const template$1 = createTemplate(
  '<div style="color:#0000;contain:strict;padding:0 var(--_pse) 0 var(--padding-left)" aria-hidden=true>'
);
const matchTemplate = createTemplate("<span> ");
const testBoundary = (str, position, pattern = /[_\p{N}\p{L}]{2}/u) => {
  if (!position)
    return false;
  return pattern.test(
    str.slice(
      position - (str.codePointAt(position - 2) > 65535 ? 2 : 1),
      position + (str.codePointAt(position) > 65535 ? 2 : 1)
    )
  );
};
const useEditorSearch = (editor, initClassName, initZIndex) => {
  return useMemo(() => {
    const nodes = [new Text()];
    const nodeValues = [];
    const container = template$1();
    const matchPositions = [];
    const stopSearch = () => {
      if (matchPositions[0]) {
        matchPositions.length = 0;
        container.remove();
      }
    };
    let regex;
    let nodeCount = 0;
    if (initClassName)
      container.className = initClassName;
    if (initZIndex)
      container.style.zIndex = initZIndex;
    return {
      search(str, caseSensitive, wholeWord, useRegExp, selection, filter, pattern) {
        if (!str)
          return stopSearch();
        if (!useRegExp)
          str = regexEscape(str);
        const value = editor.value;
        const searchStr = selection ? value.slice(...selection) : value;
        const offset = selection ? selection[0] : 0;
        let match;
        let l;
        let index;
        let i = 0;
        try {
          regex = RegExp(str, `gum${caseSensitive ? "" : "i"}`);
          while (match = regex.exec(searchStr)) {
            l = match[0].length;
            index = match.index + offset;
            if (!l)
              regex.lastIndex += value.codePointAt(index) > 65535 ? 2 : 1;
            if (wholeWord && (testBoundary(value, index, pattern) || testBoundary(value, index + l, pattern)))
              continue;
            if (!filter || filter(index, index + l))
              matchPositions[i++] = [index, index + l];
          }
        } catch (e) {
          stopSearch();
          return e.message;
        }
        if (i) {
          matchPositions.length = i;
          l = Math.min(i * 2, 2e4);
          for (i = nodes.length; i <= l; ) {
            nodes[i++] = matchTemplate();
            nodes[i++] = new Text();
          }
          for (i = nodeCount - 1; i > l; )
            nodes[i--].remove();
          if (nodeCount <= l)
            container.append(...nodes.slice(nodeCount, l + 1));
          let prevEnd = 0;
          for (i = 0; i < l; ++i) {
            const [start, end] = matchPositions[i / 2];
            const before = value.slice(prevEnd, start);
            const match2 = value.slice(start, prevEnd = end);
            if (before != nodeValues[i])
              nodes[i].data = nodeValues[i] = before;
            if (match2 != nodeValues[++i])
              nodes[i].firstChild.data = nodeValues[i] = match2;
          }
          nodes[l].data = nodeValues[l] = value.slice(prevEnd);
          if (!container.parentNode)
            editor.lines[0].append(container);
          nodeCount = l + 1;
        } else
          stopSearch();
      },
      container,
      get regex() {
        return regex;
      },
      matches: matchPositions,
      stopSearch
    };
  }, []);
};
const useEditorReplace = (editor, initClassName, initZIndex) => {
  const search = useEditorSearch(editor, initClassName, initZIndex);
  const getSelection = editor.getSelection;
  const matches = search.matches;
  const closest = () => {
    const caretPos = getSelection()[0];
    const l = matches.length;
    for (let i = l; i; ) {
      if (caretPos >= matches[--i][1])
        return (i + (matches[i][0] < caretPos)) % l;
    }
    return l ? 0 : -1;
  };
  const toggleClasses = () => {
    currentLine?.classList.toggle("match-highlight");
    currentMatch?.classList.toggle("match");
  };
  const removeSelection = () => {
    if (hasSelected) {
      toggleClasses();
      hasSelected = false;
    }
  };
  let currentLine;
  let currentMatch;
  let hasSelected = false;
  useLayoutEffect(() => {
    let removeListener = addTextareaListener(editor, "focus", removeSelection);
    return () => {
      removeListener();
      removeSelection();
    };
  }, []);
  return useMemo(
    () => Object.assign(search, {
      next() {
        const cursor = getSelection()[1];
        const l = matches.length;
        for (let i = 0, match; i < l; i++) {
          match = matches[i];
          if (match[0] - (match[0] == match[1]) >= cursor)
            return i;
        }
        return l ? 0 : -1;
      },
      prev() {
        const cursor = getSelection()[0];
        const l = matches.length;
        for (let i = l, match; i; ) {
          match = matches[--i];
          if (match[1] + (match[0] == match[1]) <= cursor)
            return i;
        }
        return l - 1;
      },
      closest,
      selectMatch(index, scrollPadding) {
        removeSelection();
        if (matches[index]) {
          setSelection(editor, ...matches[index]);
          currentLine = editor.lines[editor.activeLine];
          currentMatch = search.container.children[index];
          hasSelected = true;
          toggleClasses();
          if (currentMatch) {
            scrollToEl(editor, currentMatch, scrollPadding);
          }
        }
      },
      replace(str) {
        if (matches[0]) {
          let index = closest();
          let [start, end] = matches[index];
          let [caretStart, caretEnd] = getSelection();
          let notSelected = start != caretStart || end != caretEnd;
          if (notSelected)
            return index;
          if (editor.value.slice(start, end) == str)
            return matches[++index] ? index : 0;
          return insertText(editor, str);
        }
      },
      replaceAll(str) {
        if (!matches[0])
          return;
        let value = editor.value;
        let [start, end] = getSelection();
        let newLen = str.length;
        let newStart = start;
        let newEnd = end;
        let newValue = "";
        let l = matches.length;
        for (let i = 0; i < l; i++) {
          const [matchStart, matchEnd] = matches[i];
          const lengthDiff = newLen - matchEnd + matchStart;
          const move = (pos) => matchStart > pos ? 0 : pos >= matchEnd ? lengthDiff : lengthDiff < 0 && pos > matchStart + newLen ? newLen + matchStart - pos : 0;
          newEnd += move(end);
          newStart += move(start);
          newValue += i ? value.slice(matches[i - 1][1], matchStart) + str : str;
        }
        insertText(editor, newValue, matches[0][0], matches[l - 1][1], newStart, newEnd);
      }
    }),
    []
  );
};
const shortcut = ` (Alt+${isMac ? "Cmd+" : ""}`;
const template = createTemplate(
  `<div class=prism-search-container style=display:flex;align-items:flex-start;justify-content:flex-end><div dir=ltr class=prism-search><button type=button aria-expanded=false title="Toggle Replace" class=pce-expand></button><div spellcheck=false><div><div class="pce-input pce-find"><input autocorrect=off autocapitalize=off placeholder=Find aria-label=Find><button type=button class=prev-match title="Previous Match (Shift+Enter)"></button><button type=button class=next-match title="Next Match (Enter)"></button><div class=search-error></div></div><button type=button class=pce-close title="Close (Esc)"></button></div><div class="pce-input pce-replace"><input autocorrect=off autocapitalize=off placeholder=Replace aria-label=Replace><button type=button title=(Enter)>Replace</button><button type=button title=(${isMac ? "Cmd" : "Ctrl+Alt"}+Enter)>All</button></div><div class=pce-options><div class=pce-match-count>0<span> of </span>0</div><button type=button aria-pressed=false class=pce-regex title="RegExp Search${shortcut}R)"><span aria-hidden=true></span></button><button type=button aria-pressed=false title="Preserve Case${shortcut}P)"><span aria-hidden=true>Aa</span></button><button type=button aria-pressed=false class=pce-whole title="Match Whole Word${shortcut}W)"><span aria-hidden=true>ab</span></button><button type=button aria-pressed=false class=pce-in-selection title="Find in Selection${shortcut}L)">`
);
const toggleAttr = (el, name) => el.setAttribute(name, el.getAttribute(name) == "false");
const useSearchWidget = (editor) => {
  const replaceAPI = useEditorReplace(editor, "pce-matches");
  useEffect(() => {
    let prevLength;
    let useRegExp;
    let matchCase;
    let wholeWord;
    let searchSelection;
    let isOpen;
    let currentSelection;
    let prevUserSelection;
    let prevMargin;
    let selectNext = false;
    let marginTop;
    const searchContainer = template();
    const search = searchContainer.firstChild;
    const [toggle, div] = search.children;
    const rows = div.children;
    const [findContainer, closeEl] = rows[0].children;
    const [findInput, prevEl, nextEl, errorEl] = findContainer.children;
    const [replaceInput, replaceEl, replaceAllEl] = rows[1].children;
    const [matchCount, useRegExpEl, matchCaseEl, wholeWordEl, inSelectionEl] = rows[2].children;
    const [current, , total] = matchCount.childNodes;
    const { textarea, container, getSelection, wrapper } = editor;
    const startSearch = (selectMatch) => {
      if (selectMatch && !isWebKit)
        textarea.setSelectionRange(...prevUserSelection);
      const error = replaceAPI.search(
        findInput.value,
        matchCase,
        wholeWord,
        useRegExp,
        searchSelection
      );
      const index = error ? -1 : selectNext ? replaceAPI.next() : replaceAPI.closest();
      current.data = index + 1;
      total.data = replaceAPI.matches.length;
      findContainer.classList.toggle("pce-error", !!error);
      if (error)
        errorEl.textContent = error;
      else if (selectMatch || selectNext)
        replaceAPI.selectMatch(index, prevMargin);
    };
    const keydown = (e) => {
      if (e.keyCode >> 1 == 35 && getModifierCode(e) == (isMac ? 4 : 2)) {
        preventDefault(e);
        open();
        let [start, end] = getSelection();
        let value = editor.value;
        let word = value.slice(start, end) || value.slice(0, start).match(/[_\p{N}\p{L}]*$/u)[0] + value.slice(start).match(/^[_\p{N}\p{L}]*/u)[0];
        if (/^$|\n/.test(word))
          startSearch();
        else {
          if (useRegExp)
            word = regexEscape(word);
          document.execCommand("insertText", false, word);
          findInput.select();
        }
      }
    };
    const cleanups = [
      addTextareaListener(editor, "keydown", keydown),
      addTextareaListener(editor, "beforeinput", () => {
        if (isOpen && searchSelection)
          currentSelection = getSelection();
      }),
      editor.on("selectionChange", (selection) => {
        if (isOpen && editor.focused)
          prevUserSelection = selection;
      }),
      editor.on("update", () => {
        if (!isOpen)
          return;
        if (searchSelection && currentSelection) {
          const diff = prevLength - (prevLength = editor.value.length);
          const end = currentSelection[1];
          if (end <= searchSelection[1]) {
            searchSelection[1] -= diff;
            if (end <= searchSelection[0] - +(diff < 0))
              searchSelection[0] -= diff;
          }
        }
        startSearch();
      })
    ];
    const open = (focusInput = true) => {
      if (!isOpen) {
        isOpen = true;
        if (marginTop == null)
          prevMargin = marginTop = getStyleValue(wrapper, "marginTop");
        prevUserSelection = getSelection();
        editor.lines[0].append(searchContainer);
        updateMargin();
        resize();
        observer?.observe(container);
      }
      if (focusInput)
        findInput.select();
    };
    const close = (focusTextarea = true) => {
      if (isOpen) {
        isOpen = false;
        observer?.disconnect();
        replaceAPI.stopSearch();
        searchContainer.remove();
        updateMargin();
        if (focusTextarea)
          textarea.focus();
      }
    };
    const move = (next) => {
      if (replaceAPI.matches[0]) {
        const index = replaceAPI[next ? "next" : "prev"]();
        replaceAPI.selectMatch(index, prevMargin);
        current.data = index + 1;
      }
    };
    const updateMargin = () => {
      const newMargin = isOpen ? getStyleValue(search, "top") + getStyleValue(search, "height") : marginTop;
      const newScroll = container.scrollTop + newMargin - prevMargin;
      wrapper.style.marginTop = isOpen ? newMargin + "px" : "";
      container.scrollTop = newScroll;
      prevMargin = newMargin;
    };
    const resize = () => div.style.setProperty(
      "--search-width",
      `min(${container.clientWidth - 2}px - 2.4em - var(--padding-left),20em)`
    );
    const observer = window.ResizeObserver && new ResizeObserver(resize);
    const replace = () => {
      selectNext = true;
      const index = replaceAPI.replace(replaceInput.value);
      if (index != null) {
        current.data = index + 1;
        replaceAPI.selectMatch(index, prevMargin);
      }
      selectNext = false;
    };
    const replaceAll = () => {
      replaceAPI.replaceAll(replaceInput.value);
    };
    const keyCodeButtonMap = {
      80: matchCaseEl,
      87: wholeWordEl,
      82: useRegExpEl,
      76: inSelectionEl
    };
    const elementHandlerMap = /* @__PURE__ */ new Map([
      [nextEl, () => move(true)],
      [prevEl, move],
      [closeEl, close],
      [replaceEl, replace],
      [replaceAllEl, replaceAll],
      [
        toggle,
        () => {
          toggleAttr(toggle, "aria-expanded");
          updateMargin();
        }
      ],
      [matchCaseEl, () => matchCase = !matchCase],
      [useRegExpEl, () => useRegExp = !useRegExp],
      [wholeWordEl, () => wholeWord = !wholeWord],
      [
        inSelectionEl,
        () => {
          const value = editor.value;
          if (searchSelection)
            searchSelection = void 0;
          else {
            searchSelection = getSelection().slice(0, 2);
            if (numLines(value, ...searchSelection) > 1) {
              searchSelection = [
                getLineStart(value, searchSelection[0]),
                getLineEnd(value, searchSelection[1])
              ];
            }
          }
          prevLength = value.length;
        }
      ]
    ]);
    searchContainer.addEventListener("click", (e) => {
      const target = e.target;
      const remove = editor.on("update", () => target.focus());
      elementHandlerMap.get(target)?.();
      if (target.matches(".pce-options>button")) {
        toggleAttr(target, "aria-pressed");
        startSearch(true);
      }
      remove();
    });
    findInput.oninput = () => isOpen && startSearch(true);
    searchContainer.addEventListener("keydown", (e) => {
      const shortcut2 = getModifierCode(e);
      const target = e.target;
      const keyCode = e.keyCode;
      const isFind = target == findInput;
      if (shortcut2 == (isMac ? 5 : 1)) {
        if (keyCodeButtonMap[keyCode]) {
          preventDefault(e);
          keyCodeButtonMap[keyCode].click();
        }
      } else if (keyCode == 13 && target.tagName == "INPUT") {
        preventDefault(e);
        if (!shortcut2)
          isFind ? move(true) : replaceEl.click();
        else if (shortcut2 == 8 && isFind)
          move();
        else if (shortcut2 == (isMac ? 4 : 3) && !isFind)
          replaceAllEl.click();
        target.focus();
      } else if (!shortcut2 && keyCode == 27)
        close();
      else
        keydown(e);
    });
    editor.extensions.searchWidget = {
      open(focusInput) {
        open(focusInput);
        startSearch();
      },
      close
    };
    return () => {
      delete editor.extensions.searchWidget;
      cleanups.forEach((c) => c());
      close(false);
    };
  }, []);
};
const useHighlightSelectionMatches = (editor, caseSensitive, minLength = 1, maxLength = 200) => {
  const searchAPI = useEditorSearch(editor, "selection-matches", -1);
  useLayoutEffect(() => {
    return editor.on("selectionChange", ([start, end], value) => {
      value = editor.focused ? value.slice(start, end) : "";
      const pos = start + value.search(/\S/);
      const l = (value = value.trim()).length;
      searchAPI.search(
        minLength > l || l > maxLength ? "" : value,
        caseSensitive,
        false,
        false,
        void 0,
        (mStart, mEnd) => mStart > pos || mEnd <= pos
      );
    });
  }, [caseSensitive, minLength, maxLength]);
};
const useHighlightCurrentWord = (editor, filter, includeHyphens) => {
  const searchAPI = useEditorSearch(editor, "word-matches", -1);
  useLayoutEffect(() => {
    let noHighlight = false;
    let cleanup1 = editor.on("update", () => noHighlight = true);
    let cleanup2 = editor.on("selectionChange", ([start, end], value) => {
      if (start < end || !editor.focused || noHighlight)
        searchAPI.search("");
      else {
        let group = `[_$\\p{L}\\d${includeHyphens && includeHyphens(start) ? "-" : ""}]`;
        let before = value.slice(0, start).match(RegExp(group + "*$", "u"));
        let index = before.index;
        let word = before[0] + value.slice(start).match(RegExp("^" + group + "*", "u"))[0];
        searchAPI.search(
          /^-*(\d|$)/.test(word) || filter && !filter(index, index + word.length) ? "" : word,
          true,
          true,
          false,
          void 0,
          filter,
          RegExp(group + "{2}", "u")
        );
      }
      noHighlight = false;
    });
    return () => {
      cleanup1();
      cleanup2();
    };
  }, [filter, includeHyphens]);
};
export {
  useEditorReplace as a,
  useSearchWidget as b,
  useHighlightSelectionMatches as c,
  useHighlightCurrentWord as d,
  useEditorSearch as u
};
//# sourceMappingURL=selection-C7vs8aTI.js.map
